package com.boman.system.mapper;

import java.util.List;

import org.apache.ibatis.annotations.Param;

import com.boman.domain.SysDept;

/**
 * 部门管理 数据层
 * 
 * @author ruoyi
 */
public interface SysDeptMapper
{
    /**
     * 查询部门管理数据
     * 
     * @param dept 部门信息
     * @return 部门信息集合
     */
    public List<SysDept> selectDeptList(SysDept dept);

    /**
     * 查询部门全部数据,不带权限
     * @return
     */
    public List<SysDept> selectDeptsList();

    /**
     * 根据角色ID查询部门树信息
     * 
     * @param id 角色ID
     * @param deptCheckStrictly 部门树选择项是否关联显示
     * @return 选中部门列表
     */
    public List<Integer> selectDeptListById(@Param("id") Long id, @Param("deptCheckStrictly") boolean deptCheckStrictly);


    public List<SysDept> validateHasChirld(@Param("depts") List<SysDept> depts);

    /**
     * 根据部门ID查询信息
     * 
     * @param id 部门ID
     * @return 部门信息
     */
    public SysDept selectDeptById(Long id);

    /**
     * 根据部门名称查询信息
     * @param deptName
     * @return
     */
    public SysDept selectDeptByDeptName(String  deptName);

    /**
     * 根据ID查询所有子部门
     * 
     * @param id 部门ID
     * @return 部门列表
     */
    public List<SysDept> selectChildrenDeptById(Long id);


    List<SysDept> getByParentId(Long id);

    /**
     * 根据ID查询所有子部门（正常状态）
     * 
     * @param id 部门ID
     * @return 子部门数
     */
    public int selectNormalChildrenDeptById(Long id);

    /**
     * 是否存在子节点
     * 
     * @param id 部门ID
     * @return 结果
     */
    public int hasChildById(Long id);

    /**
     * 查询部门是否存在用户
     * 
     * @param id 部门ID
     * @return 结果
     */
    public int checkDeptExistUser(Long id);

    /**
     * 校验部门名称是否唯一
     * 
     * @param deptName 部门名称
     * @param parentId 父部门ID
     * @return 结果
     */
    public SysDept checkDeptNameUnique(@Param("deptName") String deptName, @Param("parentId") Long parentId);

    /**
     * 新增部门信息
     * 
     * @param dept 部门信息
     * @return 结果
     */
    public int insertDept(SysDept dept);

    /**
     * 修改部门信息
     * 
     * @param dept 部门信息
     * @return 结果
     */
    public int updateDept(SysDept dept);

    /**
     * 修改所在部门的父级部门状态
     * 
     * @param dept 部门
     */
    public void updateDeptStatus(SysDept dept);

    /**
     * 修改子元素关系
     * 
     * @param depts 子元素
     * @return 结果
     */
    public int updateDeptChildren(@Param("depts") List<SysDept> depts);

    /**
     * 删除部门管理信息
     * 
     * @param id 部门ID
     * @return 结果
     */
    public int deleteDeptById(Long id);

    /**
     * 功能描述: 只找父亲，不招爷爷
     *
     * @param parentId parentId
     * @return com.boman.domain.SysDept
     */
    SysDept getParentById(Long parentId);

    List<SysDept> selectByAreaId(String areaId);
}
